(function (global, factory) {
    typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports, require('rxjs'), require('micro-dash'), require('rxjs/operators')) :
    typeof define === 'function' && define.amd ? define('s-rxjs-utils', ['exports', 'rxjs', 'micro-dash', 'rxjs/operators'], factory) :
    (global = global || self, factory(global['s-rxjs-utils'] = {}, global.rxjs, global.microDash, global.rxjs.operators));
}(this, function (exports, rxjs, microDash, operators) { 'use strict';

    /*! *****************************************************************************
    Copyright (c) Microsoft Corporation. All rights reserved.
    Licensed under the Apache License, Version 2.0 (the "License"); you may not use
    this file except in compliance with the License. You may obtain a copy of the
    License at http://www.apache.org/licenses/LICENSE-2.0

    THIS CODE IS PROVIDED ON AN *AS IS* BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
    KIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION ANY IMPLIED
    WARRANTIES OR CONDITIONS OF TITLE, FITNESS FOR A PARTICULAR PURPOSE,
    MERCHANTABLITY OR NON-INFRINGEMENT.

    See the Apache Version 2.0 License for specific language governing permissions
    and limitations under the License.
    ***************************************************************************** */

    function __read(o, n) {
        var m = typeof Symbol === "function" && o[Symbol.iterator];
        if (!m) return o;
        var i = m.call(o), r, ar = [], e;
        try {
            while ((n === void 0 || n-- > 0) && !(r = i.next()).done) ar.push(r.value);
        }
        catch (error) { e = { error: error }; }
        finally {
            try {
                if (r && !r.done && (m = i["return"])) m.call(i);
            }
            finally { if (e) throw e.error; }
        }
        return ar;
    }

    function __spread() {
        for (var ar = [], i = 0; i < arguments.length; i++)
            ar = ar.concat(__read(arguments[i]));
        return ar;
    }

    /**
     * @fileoverview added by tsickle
     * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
     */
    /**
     * 1. Caches the last value emitted to give to new subscribers (without running any upstream pipe operators)
     * 2. Manages all subscribers directly, without passing subscriptions up the stream.
     *
     * This is very similar to `shareReplay(1)`, except that once all subscribers unsubscribe this also unsubscribes from the upstream observable.
     *
     * ```ts
     * const source = new BehaviorSubject(1000);
     * const result = source.pipe(expensiveComputation(), cache());
     * source.subscribe(); // expensiveComputation(1000) runs
     * source.subscribe(); // the cached result is used
     * source.next(2000); // expensiveComputation(2000) runs once, emitted to both subscribers
     * ```
     * @template T
     * @return {?}
     */
    function cache() {
        return (/**
         * @param {?} source
         * @return {?}
         */
        function (source) {
            /** @type {?} */
            var middleMan = new rxjs.ReplaySubject(1);
            /** @type {?} */
            var upstreamSubscription;
            return new rxjs.Observable((/**
             * @param {?} subscriber
             * @return {?}
             */
            function (subscriber) {
                if (middleMan.observers.length === 0) {
                    upstreamSubscription = source.subscribe(middleMan);
                }
                /** @type {?} */
                var subscription = middleMan.subscribe(subscriber);
                // teardown logic
                return (/**
                 * @return {?}
                 */
                function () {
                    subscription.unsubscribe();
                    if (middleMan.observers.length === 0) {
                        upstreamSubscription.unsubscribe();
                    }
                });
            }));
        });
    }

    /**
     * @fileoverview added by tsickle
     * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
     */
    /**
     * Use this to create a complex pipeable operator. It is usually better style to compose existing operators than to create a brand new one, but when you need full control this can reduce some boilerplate.
     *
     * The supplied `subscriber` will act as a simple pass-through of all values, errors, and completion to `destination`. Modify it for your needs.
     *
     * A simple example, recreating the "map" operator:
     * ```ts
     * function map<I, O>(fn: (input: I) => O) {
     *   return createOperatorFunction<I, O>(
     *     (subscriber, destination) => {
     *       subscriber.next = (value) => {
     *         destination.next(fn(value));
     *       };
     *     },
     *   );
     * }
     * ```
     *
     * For a more complex example, check the source of `skipAfter`.
     * @template SourceType, DestinationType
     * @param {?} modifySubscriber
     * @return {?}
     */
    function createOperatorFunction(modifySubscriber) {
        return (/**
         * @param {?} source
         * @return {?}
         */
        function (source) {
            return new rxjs.Observable((/**
             * @param {?} destination
             * @return {?}
             */
            function (destination) {
                /** @type {?} */
                var subscriber = new rxjs.Subscriber(destination);
                modifySubscriber(subscriber, destination);
                return source.subscribe(subscriber);
            }));
        });
    }

    /**
     * @fileoverview added by tsickle
     * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
     */
    /**
     * Works like `filter()`, but always lets through the first emission for each new subscriber. This makes it suitable for subscribers that expect the observable to behave like a `BehaviorSubject`, where the first emission is processed synchronously during the call to `subscribe()` (such as the `async` pipe in an Angular template).
     *
     * ```
     * source:                   |-false--true--false--true--false--true-|
     * filterBehavior(identity): |-false--true---------true---------true-|
     * filterBehavior(identity):        |-true---------true---------true-|
     * filterBehavior(identity):              |-false--true---------true-|
     * ```
     * @template T
     * @param {?} predicate
     * @return {?}
     */
    function filterBehavior(predicate) {
        return createOperatorFunction((/**
         * @param {?} subscriber
         * @param {?} destination
         * @return {?}
         */
        function (subscriber, destination) {
            /** @type {?} */
            var firstValue = true;
            subscriber.next = (/**
             * @param {?} value
             * @return {?}
             */
            function (value) {
                if (firstValue || predicate(value)) {
                    destination.next(value);
                    firstValue = false;
                }
            });
        }));
    }

    /**
     * @fileoverview added by tsickle
     * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
     */
    /**
     * @template UpstreamType, DownstreamType
     * @param {?} buildCacheKey
     * @param {?} buildDownstreamItem
     * @return {?}
     */
    function mapAndCacheElements(buildCacheKey, buildDownstreamItem) {
        /** @type {?} */
        var cache = new Map();
        return operators.map((/**
         * @param {?} upstreamItems
         * @return {?}
         */
        function (upstreamItems) {
            /** @type {?} */
            var nextCache = new Map();
            /** @type {?} */
            var downstreamItems = microDash.map(upstreamItems, (/**
             * @param {?} upstreamItem
             * @param {?} key
             * @return {?}
             */
            function (upstreamItem, key) {
                /** @type {?} */
                var cacheKey = buildCacheKey(upstreamItem, key);
                /** @type {?} */
                var downstreamItem;
                if (cache.has(cacheKey)) {
                    downstreamItem = (/** @type {?} */ (cache.get(cacheKey)));
                }
                else if (nextCache.has(cacheKey)) {
                    downstreamItem = (/** @type {?} */ (nextCache.get(cacheKey)));
                }
                else {
                    downstreamItem = buildDownstreamItem(upstreamItem, key);
                }
                nextCache.set(cacheKey, downstreamItem);
                return downstreamItem;
            }));
            cache = nextCache;
            return downstreamItems;
        }));
    }

    /**
     * @fileoverview added by tsickle
     * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
     */
    /**
     * Applies `buildDownstreamItem` to each item in the upstream array and emits the result. Each downstream item is cached using the key generated by `buildCacheKey` so that the next emission contains references to the matching objects from the previous emission, without running `buildDownstreamItem` again. The cache is only held between successive emissions.
     *
     * This is useful e.g. when using the result in an `*ngFor` expression of an angular template, to prevent angular from rebuilding the inner component and to allow `OnPush` optimizations in the inner component.
     *
     * If multiple items in an upstream array have the same cache key, it will only call `buildDownstreamItem` once.
     *
     * ```ts
     * const mapWithCaching = mapAndCacheElements(
     *   (item) => item,
     *   (item) => item + 1
     * )
     * ```
     * ```
     * source:         -[1, 2]---[1, 2, 3]---[2]--|
     * mapWithCaching: -[2, 3]---[2, 3, 4]---[3]--|
     * ```
     * @type {?}
     */
    var mapAndCacheArrayElements = (/** @type {?} */ (mapAndCacheElements));

    /**
     * @fileoverview added by tsickle
     * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
     */
    /**
     * Applies `buildDownstreamItem` to each item in the upstream object and emits an array containing the results. Each downstream item is cached using the key generated by `buildCacheKey` so that the next emission contains references to the matching objects from the previous emission, without running `buildDownstreamItem` again. The cache is only held between successive emissions.
     *
     * This is useful e.g. when using the result in an `*ngFor` expression of an angular template, to prevent angular from rebuilding the inner component and to allow `OnPush` optimizations in the inner component.
     *
     * If multiple items in an upstream object have the same cache key, it will only call `buildDownstreamItem` once.
     *
     * ```ts
     * const mapWithCaching = mapAndCacheElements(
     *   (item, key) => key,
     *   (item, key) => item + 1
     * )
     * ```
     * ```
     * source:         -{ a: 1, b: 2 }---{ a: 1, b: 2, c: 3 }---{ b: 2 }--|
     * mapWithCaching: -[2, 3]-----------[2, 3, 4]--------------[3]--|
     * ```
     * @type {?}
     */
    var mapAndCacheObjectElements = (/** @type {?} */ (mapAndCacheElements));

    /**
     * @fileoverview added by tsickle
     * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
     */
    /**
     * Causes the next value in the pipe to be skipped after `skip$` emits a value. For example:
     *
     * ```
     * source: -1-----2-----3-----4-----5-|
     * skip$:  ----0----------0-0----------
     *
     * result: -1-----------3-----------5-|
     * ```
     * ```ts
     * const source = new Subject();
     * const skip$ = new Subject();
     * const result = source.pipe(skipAfter(skip$));
     *
     * source.next(1); // result emits `1`
     *
     * skip$.next();
     * source.next(2); // result does not emit
     * source.next(3); // result emits `3`
     *
     * skip$.next();
     * skip$.next();
     * source.next(4); // result does not emit
     * source.next(5); // result emits `5`
     * ```
     * @template T
     * @param {?} skip$
     * @return {?}
     */
    function skipAfter(skip$) {
        return createOperatorFunction((/**
         * @param {?} subscriber
         * @param {?} destination
         * @return {?}
         */
        function (subscriber, destination) {
            /** @type {?} */
            var skipNext = false;
            subscriber.add(skip$.subscribe((/**
             * @return {?}
             */
            function () {
                skipNext = true;
            })));
            subscriber.next = (/**
             * @param {?} value
             * @return {?}
             */
            function (value) {
                if (skipNext) {
                    skipNext = false;
                }
                else {
                    destination.next(value);
                }
            });
        }));
    }

    /**
     * @fileoverview added by tsickle
     * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
     */
    /**
     * Emits the upstream value as the first element in an array, followed by the last `count` values in reverse chronological order.
     *
     * ```
     * source:         -1----2------3--------4--------5--------|
     * withHistory(2): -[1]--[2,1]--[3,2,1]--[4,3,2]--[5,4,3]--|
     * withHistory(0): -[1]--[2]----[3]------[4]------[5]------|
     * ```
     * @template T
     * @param {?} count
     * @return {?}
     */
    function withHistory(count) {
        return operators.scan((/**
         * @param {?} buf
         * @param {?} value
         * @return {?}
         */
        function (buf, value) { return __spread([value], buf.slice(0, count)); }), []);
    }

    /**
     * @fileoverview added by tsickle
     * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
     */

    /**
     * @fileoverview added by tsickle
     * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
     */
    /**
     * Tracks all subscriptions to easily unsubscribe from them all during cleanup. Also binds callbacks to `this` for convenient use as a superclass, e.g.:
     *
     * ```ts
     * class EventLogger extends SubscriptionManager {
     *   constructor(private prefix: string, event$: Observable<string>) {
     *     super();
     *
     *     // you can pass in an instance method here and it will be bound to `this`
     *     this.subscribeTo(event$, this.log);
     *   }
     *
     *   log(event: string) {
     *     // even though this is used as a callback, you can still use `this`
     *     console.log(this.prefix + event);
     *   }
     * }
     * ```
     */
    var   /**
     * Tracks all subscriptions to easily unsubscribe from them all during cleanup. Also binds callbacks to `this` for convenient use as a superclass, e.g.:
     *
     * ```ts
     * class EventLogger extends SubscriptionManager {
     *   constructor(private prefix: string, event$: Observable<string>) {
     *     super();
     *
     *     // you can pass in an instance method here and it will be bound to `this`
     *     this.subscribeTo(event$, this.log);
     *   }
     *
     *   log(event: string) {
     *     // even though this is used as a callback, you can still use `this`
     *     console.log(this.prefix + event);
     *   }
     * }
     * ```
     */
    SubscriptionManager = /** @class */ (function () {
        function SubscriptionManager() {
            this.subscriptions = new rxjs.Subscription();
        }
        /**
         * @template T
         * @param {?} observable
         * @param {?=} next
         * @param {?=} error
         * @param {?=} complete
         * @return {?}
         */
        SubscriptionManager.prototype.subscribeTo = /**
         * @template T
         * @param {?} observable
         * @param {?=} next
         * @param {?=} error
         * @param {?=} complete
         * @return {?}
         */
        function (observable, next, error, complete) {
            this.subscriptions.add(observable.subscribe(this.bind(next), this.bind(error), this.bind(complete)));
        };
        /**
         * @return {?}
         */
        SubscriptionManager.prototype.unsubscribe = /**
         * @return {?}
         */
        function () {
            this.subscriptions.unsubscribe();
            this.subscriptions = new rxjs.Subscription();
        };
        /**
         * @private
         * @param {?=} fn
         * @return {?}
         */
        SubscriptionManager.prototype.bind = /**
         * @private
         * @param {?=} fn
         * @return {?}
         */
        function (fn) {
            return fn && fn.bind(this);
        };
        return SubscriptionManager;
    }());

    exports.SubscriptionManager = SubscriptionManager;
    exports.cache = cache;
    exports.createOperatorFunction = createOperatorFunction;
    exports.filterBehavior = filterBehavior;
    exports.mapAndCacheArrayElements = mapAndCacheArrayElements;
    exports.mapAndCacheObjectElements = mapAndCacheObjectElements;
    exports.skipAfter = skipAfter;
    exports.withHistory = withHistory;

    Object.defineProperty(exports, '__esModule', { value: true });

}));
//# sourceMappingURL=s-rxjs-utils.umd.js.map
